@@ -6,7 +6,7 @@ import sys
 
 import click
 
-from aicrowd.contexts import pass_config, ConfigContext
+from aicrowd.contexts import ConfigContext, pass_config
 from aicrowd.utils import AliasedGroup
 from aicrowd.utils.utils import exception_handler
 
