@@ -5,6 +5,7 @@ Class describing a challenge
 import logging
 
 from git import Repo
+from git.exc import InvalidGitRepositoryError
 
 from aicrowd.constants import ChallengeConstants
 from aicrowd.errors import CONFIG_NOT_SET
@@ -17,15 +18,20 @@ class Challenge:
     """
 
     def __init__(self):
+        # TODO: implement __repr__/__str__ for this
         self.log: logging.Logger = logging.getLogger()
         self.info = None
+        try:
+            self.repo = Repo(".", search_parent_directories=True)
+        except InvalidGitRepositoryError:
+            self.repo = None
 
     def load(self):
         """
         Check git config for CLI stuff
         """
         try:
-            with Repo(".").config_reader() as git_conf:
+            with self.repo.config_reader() as git_conf:
                 challenge_id = git_conf.get_value(
                     ChallengeConstants.CONFIG_SECTION_NAME,
                     ChallengeConstants.CONFIG_ID_KEY,
