@@ -4,7 +4,7 @@ For click contexts
 
 import click
 
-from aicrowd.contexts import CLIConfig, Challenge
+from aicrowd.contexts import Challenge, CLIConfig
 
 
 class ConfigContext:
