@@ -4,7 +4,7 @@
 # this will never get called unless you are using ipython (or jupyter)
 # since they will already have IPython installed, there's no need to add this as a dependency
 # pylint: disable=E0401
-from IPython.core.magic import Magics, magics_class, line_magic
+from IPython.core.magic import Magics, line_magic, magics_class
 
 # pylint: enable=E0401
 from aicrowd.cli import cli
