@@ -33,3 +33,9 @@ class SubmissionUploadException(SubmissionException):
     """
     Something went wrong while uploading the file
     """
+
+
+class SubmissionGitException(SubmissionException):
+    """
+    Something went wrong while doing some git operation
+    """
