@@ -6,10 +6,13 @@ import logging
 import os
 import re
 import shutil
+from json.decoder import JSONDecodeError
 from pathlib import Path
-from typing import Tuple
+from typing import Tuple, Union
 
 import requests
+from git import Remote, Repo
+from git.exc import GitError
 from requests_toolbelt import MultipartEncoderMonitor
 from rich import box
 from rich.console import Console
@@ -23,19 +26,25 @@ from rich.progress import (
     TransferSpeedColumn,
 )
 from rich.table import Table
+from slugify import slugify
 
 # pylint: disable=unused-import
 from aicrowd.challenge.helpers import must_get_api_key, parse_cli_challenge
 
 # pylint: enable=unused-import
-from aicrowd.constants import RAILS_HOST
+from aicrowd.constants import AICROWD_GITLAB_HOST, RAILS_HOST
+from aicrowd.errors import INVALID_CHALLENGE_REPO
 from aicrowd.submission.exceptions import (
     ChallengeNotFoundException,
+    SubmissionException,
     SubmissionFileException,
+    SubmissionGitException,
     SubmissionUploadException,
 )
+from aicrowd.utils.git import ls_remote
 from aicrowd.utils.jupyter import is_jupyter
 from aicrowd.utils.utils import zip_fs_path
+from aicrowd_api.challenge import check_registration_status
 from aicrowd_api.submission import (
     create_rails_submission,
     get_submission_upload_details,
@@ -183,7 +192,7 @@ class S3Uploader:
             )
 
 
-def upload_submission(host: str, fields: dict, file_path: str) -> str:
+def upload_submission(host: str, fields: dict, file_path: str) -> Union[str, None]:
     """
     uploads a file to s3 using presigned url details
 
@@ -370,6 +379,206 @@ def submit_file(
     return response.get("data")
 
 
+def find_aicrowd_git_remote(repo: Repo) -> Remote:
+    """
+    Finds the appropriate git remote to push the submission to
+
+    Checks whether `aicrowd` exists. then, finds any remote which points to AIcrowd gitlab
+    """
+    # check if `aicrowd` remote exists
+    for remote in repo.remotes:
+        if remote.name == "aicrowd":
+            return remote
+
+    # return the first remote with aicrowd gitlab as remote
+    for remote in repo.remotes:
+        if next(filter(lambda url: AICROWD_GITLAB_HOST in url, remote.urls), None):
+            return remote
+
+    raise SubmissionUploadException(
+        "No git remote found",
+        "Please re-create the repository with `aicrowd challenge init`",
+        INVALID_CHALLENGE_REPO,
+    )
+
+
+def generate_submission_tag(repo: Repo, submission_description: str) -> str:
+    """
+    Generates a submission tag
+
+    checks existing tags and creates a new one
+
+    Args:
+        repo: the git repo object for this challenge
+        submission_description: description for this submission
+
+    Returns:
+        the (hopefully) non-conflicting generated tag
+    """
+    # TODO: what if the next tag exists on the remote but not here?
+
+    tag_prefix = "submission-"
+
+    base_tag = slugify(
+        submission_description
+        if submission_description.startswith(tag_prefix)
+        else f"{tag_prefix}{submission_description}",
+        regex_pattern=r"[^-a-z0-9.]+",
+    )
+
+    tag_already_exists = False
+
+    for tag in repo.tags:
+        if tag.name == base_tag:
+            tag_already_exists = True
+            break
+
+    if not tag_already_exists:
+        return base_tag
+
+    largest_version = -1
+    cli_tag_re = re.compile(f"^{base_tag}-v" + r"\d+$")
+
+    for tag in repo.tags:
+        if cli_tag_re.match(tag.name):
+            tag_version = int(tag.name.split("-v")[1])
+            if tag_version > largest_version:
+                largest_version = tag_version
+
+    return f"{base_tag}-v{largest_version+1}"
+
+
+def submit_git(
+    repo: Repo,
+    submission_message: str,
+    submission_tag: str = None,
+    print_links: bool = False,
+):
+    """
+    Git based submissions
+
+    the source code is committed and a tag starting with 'submission-' is pushed
+
+    Args:
+        repo: the git repo object for this challenge
+        submission_message: this will be used as the commit message
+        submission_tag: this will be used to name the tag
+    """
+    remote = find_aicrowd_git_remote(repo)
+
+    if submission_tag is None:
+        submission_tag = generate_submission_tag(repo, submission_message)
+    if not submission_tag.startswith("submission-"):
+        submission_tag = f"submission-{submission_tag}"
+
+    if repo.is_dirty() or len(repo.untracked_files) > 0:
+        # add everything from the repo root
+        # assumes that gitignore has been set properly
+        try:
+            repo.git.add(all=True)
+            repo.index.commit(submission_message)
+        except GitError as e:
+            raise SubmissionGitException(
+                str(e),
+                "Couldn't commit files. Is the git working tree in an unstable state?",
+            ) from e
+
+    try:
+        repo.create_tag(submission_tag)
+    except GitError as e:
+        raise SubmissionGitException(str(e), "Couldn't create git tag") from e
+
+    try:
+        # this is needed to push the objects in case it is a new repo
+        remote.push()
+        # this actually pushes the tag
+        remote.push(submission_tag)
+
+        remote_url = next(remote.urls)
+
+        # check if submission was successfully pushed
+        assert f"refs/tags/{submission_tag}" in ls_remote(
+            remote_url
+        ), "Submission push failed, tag not found in the remote"
+    except (GitError, AssertionError) as e:
+        raise SubmissionGitException(
+            str(e),
+            "Couldn't push local changes. Has the remote repository been updated?",
+        ) from e
+
+    # not pointing to the exact issue, but to the issue list page
+    # to get the exact issue, we'll need gitlab API key
+    if print_links:
+        gitlab_repo = remote_url.split(AICROWD_GITLAB_HOST)[1][1:]
+        if gitlab_repo.endswith(".git"):
+            gitlab_repo = gitlab_repo[:-4]
+
+        gitlab_issues_url = f"https://{AICROWD_GITLAB_HOST}/{gitlab_repo}/-/issues"
+
+        print(f"Check your submission status at {gitlab_issues_url}")
+
+
+def verify_aicrowd_json():
+    aicrowd_json = Path("aicrowd.json")
+    if not aicrowd_json.is_file():
+        raise SubmissionException(
+            "aicrowd.json doesn't exist",
+            "All git submissions should have the `aicrowd.json` file. "
+            "Please clone the starter kit again",
+        )
+
+    try:
+        aicrowd_json_dict = json.loads(aicrowd_json.read_text())
+    except JSONDecodeError as e:
+        raise SubmissionException(
+            "aicrowd.json is not a valid json file",
+            "Please clone the starter kit again",
+        ) from e
+
+    required_keys_types = [
+        ("grader_id", str),
+        ("challenge_id", str),
+        ("authors", list),
+        ("gpu", bool),
+        ("description", str),
+    ]
+    for key, value_type in required_keys_types:
+        if key not in aicrowd_json_dict or not isinstance(
+            aicrowd_json_dict[key], value_type
+        ):
+            raise SubmissionException(
+                f"Expected key {key} of type {value_type}",
+                "Some keys are missing or have the wrong value type. "
+                "Please refer to the original starter kit",
+            )
+
+
+def verify_participant_registration(api_key: str, challenge_id: int):
+    """
+    Checks whether participant has registered for the challenge or not
+    """
+    if not check_registration_status(api_key, challenge_id):
+        raise SubmissionException(
+            "You haven't registered for this challenge",
+            "Please go to the challenge homepage and register",
+        )
+
+
+def verify_file_submission(api_key: str, challenge_id: int):
+    """
+    Checks for validity of file submissions
+    """
+    verify_participant_registration(api_key, challenge_id)
+
+
+def verify_git_submission(api_key: str, challenge_id: int):
+    """
+    Checks for validity of git submission
+    """
+    verify_participant_registration(api_key, challenge_id)
+    verify_aicrowd_json()
+
+
 def zip_assets(assets_dir: str, target_zip_path: str):
     """
     Zip the files under assets directory
