@@ -6,6 +6,20 @@ from rich.console import Console
 from aicrowd.constants import JupyterConfigConstants
 
 
+def is_ipykernel_present():
+    """
+    Check if process is running inside IPython kernel
+    """
+    try:
+        # pylint: disable=import-error,no-name-in-module
+        from IPython import get_ipython
+
+        ipy_kernel = get_ipython()
+        return ipy_kernel is not None
+    except ImportError:
+        return False
+
+
 def get_ipython_object():
     # this will never succeed unless you are using ipython (or jupyter)
     # since they will already have IPython installed, there's no need to
